package remote;

import java.util.StringTokenizer;
import java.util.*;
import java.io.*;

public class Command implements Serializable
{
	private String _sCommand = null;
	private boolean _bIsBinary = false;
	private String _sFilename = null;
	private String _sDirectory = null;
	private String _sFiletype = null;

	private byte[] _transferFile = null;

	public static final String BINARY = "binary";
	public static final String COPY = "copy";
	public static final String ANT = "ant";
	public static final String EXECUTE = "execute";
	public static final String DONE = "done";

	public Command(String sCommand)
	{
		_sCommand = sCommand;
	}

	public Command(String sCommand, String sFiletype, String sDirectory, String sFilename)
		throws CommandException
	{
		// set class variables
		_sCommand = sCommand;
		_sFiletype = sFiletype;
		_sDirectory = sDirectory;
		_sFilename = sFilename;

		// setup ascii/binary flag
		if ( _sFiletype != null)
		{
			_bIsBinary = _sFiletype.equals(BINARY);
		}

		try
		{
			// load file into byte array
			loadFile( _sDirectory, _sFilename);
		}
		catch (IOException ioe)
		{
			throw new CommandException("loadFile", ioe);
		}
	}

	public String getCommand() throws CommandException
	{
		return _sCommand;
	}

	public boolean isBinary() throws CommandException
	{
		if ( _sFiletype == null)
		{
			throw new CommandException("No filetype given");
		}

		return _bIsBinary;
	}

	public String getFilename() throws CommandException
	{
		return _sFilename;
	}

	public String getDirectory() throws CommandException
	{
		return _sDirectory;
	}

	private void loadFile(String directory, String filename)
		throws IOException
	{
		readFile( directory +"/"+ filename);
	}

	public byte[] getByteArray()
	{
		return _transferFile;
	}

	// TO TRANSFER BINARY FILES

	private void readFile(String sFullFilename) throws IOException
	{
		RandomAccessFile raf = new RandomAccessFile( sFullFilename, "r");

		long size = raf.length();
		Integer myInt = new Integer( size +"" );

		int iSize = myInt.intValue();

		System.out.println("RCT size L("+ size +") I("+ iSize +")");

		_transferFile = new byte[iSize];

		raf.read(_transferFile);
		raf.close();
	}

	private void writeFile(String filename) throws IOException
	{
		File file = new File(filename);

		RandomAccessFile raf = new RandomAccessFile( file, "rw" );

		System.out.println("RCT bytes "+ _transferFile.length);

		raf.write(_transferFile, 0, _transferFile.length);
		raf.close();
	}

	public static void main(String[] args)
	{
		try
		{
			Command c = new Command("copy","binary","dir","filename");

			System.out.println( c.getCommand() );
			System.out.println( c.isBinary() );
			System.out.println( c.getDirectory() );
			System.out.println( c.getFilename() );

			c = new Command("ant");

			System.out.println( c.getCommand() );
			System.out.println( c.isBinary() );
			System.out.println( c.getDirectory() );
			System.out.println( c.getFilename() );

			c = new Command("copy");

			System.out.println( c.getCommand() );
			System.out.println( c.isBinary() );
			System.out.println( c.getDirectory() );
			System.out.println( c.getFilename() );

		}
		catch ( CommandException ce)
		{
			ce.printStackTrace();
		}
	}
}