/**
From: http://developer.java.sun.com/developer/TechTips/1999/tt1021.html#tip1

--------------------------------------------------------------------------------
CREATING IMAGE THUMBNAILS

This tip demonstrates how you can create a small thumbnail version of a larger image. This is especially important for web applications that need to display a number of images on a page and do it in a reasonable amount of time. 

The tip applies to the JavaTM 2 Platform, JDKTM v1.2 and uses the com.sun.image.codec.jpeg package. Although this package is currently included in JDK v1.2, it's not yet part of the Java 2 Platform APIs and so it might change or be deleted in the future. 

The example program below illustrates how to create an image thumbnail. The program takes any GIF or JPEG file and creates a smaller JPEG file. The program shrinks the original image so that it fits into a square whose size you specify. In other words, the program maintains the aspect ratio, that is, the width-to-height ratio, of the original image. It also ensures that the width and height of the thumbnail are less than the size you specify. 

Here's the code for the example program. 
*/

package training.ant.task;

import java.awt.Image;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.FileOutputStream;
import java.io.File;
import java.io.FilenameFilter;
import javax.swing.ImageIcon;
import com.sun.image.codec.jpeg.JPEGCodec;
import com.sun.image.codec.jpeg.JPEGImageEncoder;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;

public class ThumbnailizeTask extends Task {

	static boolean DEBUG=true;
	
	static void trace(String msg){
		if(DEBUG) System.out.println(msg);
	}

	
	private int maxdim=128;
	private String dir;
	private String file;
	private String toDir;
	private String toFile;
	
	
	public void execute() throws BuildException {
		if(file==null && dir==null) {
			throw new BuildException("No file or dir attributes");
		}
		if(file!=null && dir!=null) {
			throw new BuildException("Cant use file or dir attributes both");
		}
		
		if((file !=null && dir==null) && (toDir==null && toFile==null)){
			throw new BuildException("Attributes toFile or toDir are required");
		}
		if((file ==null && dir!=null) && (toDir==null)){
			throw new BuildException("Attribute toDir is required");
		}
		
		if(dir!=null){
			File file1=new File(dir);
			File file2=new File(toDir);
			if(file1.isDirectory() && file2.isDirectory()){
				FilenameFilter iff=new ImageFilenameFilter();
				String[] filenamelist=file1.list(iff);
				for(int i=0;i<filenamelist.length;i++){
					String f1=new File(dir,filenamelist[i]).getAbsolutePath();
					String f2=new File(toDir,filenamelist[i]).getAbsolutePath();					
					trace(f1+"-->"+f2);
					createThumbnail(f1, f2, maxdim);
				}
			} else {
				throw new BuildException("Attributes dir or toDir are not repertories");	
			}
		}
		
		if(file!=null){
			File file1=new File(file);
			if(!file1.isFile()){
				throw new BuildException("Attribute file does not exist or is not a file");
			}
 
			
			if(toFile!=null){
				trace(file+"-->"+toFile);
				createThumbnail(file, toFile, maxdim);
			} else {
				File file2=new File(toDir);
				if(!file2.isDirectory()){
					throw new BuildException("Attribute toDir does not exist or is not a repertory");	
				}
				String f2=new File(toDir,file1.getName()).getAbsolutePath();								
				trace(file+"-->"+f2);
				createThumbnail(file, f2, maxdim);
			}
		}
	}

    /**
     * Sets the maximum width or heigth of the thumbnail.
     */
	public void setMaxdim(int maxdim) {
		this.maxdim = maxdim;
	}

    /**
     * Sets a single source file to thumbnailize.
     */
	public void setFile(String fileName) {
		this.file = fileName;
	}
	
    /**
     * Sets a source directory to thumbnailize.
     */
	public void setDir(String dirName) {
		this.dir = dirName;
	}
	
    /**
     * Sets the destination directory.
     */
	public void setTodir(String dirName) {
		this.toDir = dirName;
	}
	
    /**
     * Sets the destination file.
     */
	public void setTofile(String fileName) {
		this.toFile = fileName;
	}
	
	/**
	 * Reads an image in a file and creates 
	 * a thumbnail in another file.
	 * @param orig The name of image file.
	 * @param thumb The name of thumbnail file.  
	 * Will be created if necessary.
	 * @param maxDim The width and height of 
	 * the thumbnail must 
	 * be maxDim pixels or less.
	 */
	public static void createThumbnail(String orig, String thumb, int maxDim) {
		try {
			// Get the image from a file.
			Image inImage = new ImageIcon(orig).getImage();

			// Determine the scale.
			double scale = (double)maxDim/(
			                       double)inImage.getHeight(null);
			if (inImage.getWidth(
			            null) > inImage.getHeight(null)) {
				scale = (double)maxDim/(
				                double)inImage.getWidth(null);
			}

			// Determine size of new image.
			//One of them
			// should equal maxDim.
			int scaledW = (int)(
			                      scale*inImage.getWidth(null));
			int scaledH = (int)(
			                      scale*inImage.getHeight(null));

			// Create an image buffer in
			//which to paint on.
			BufferedImage outImage =
			        new BufferedImage(scaledW, scaledH,
			                          BufferedImage.TYPE_INT_RGB);

			// Set the scale.
			AffineTransform tx =
			        new AffineTransform();

			// If the image is smaller than
			//the desired image size,
			// don't bother scaling.
			if (scale < 1.0d) {
				tx.scale(scale, scale);
			}

			// Paint image.
			Graphics2D g2d =
			        outImage.createGraphics();
			g2d.drawImage(inImage, tx, null);
			g2d.dispose();

			// JPEG-encode the image
			//and write to file.
			OutputStream os =
			        new FileOutputStream(thumb);
			JPEGImageEncoder encoder =
			        JPEGCodec.createJPEGEncoder(os);
			encoder.encode(outImage);
			os.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}



}

class ImageFilenameFilter implements FilenameFilter {
	public boolean accept(File dir, String name) {
		if(
		        name.indexOf(".gif")!=-1 || name.indexOf(".jpg")!=-1 || name.indexOf(".jpeg")!=-1
		        || name.indexOf(".GIF")!=-1 || name.indexOf(".JPG")!=-1 || name.indexOf(".JPEG")!=-1
		) {
			return true;
		} else {
			return false;
		}
	}
}
